%   Description: Takes a list of points sorted by their x-coordinates and
%   computes the convex hull in linear time.
%   Reference: Graham, RL, An efficient algorithm for determining the
%   convex hull of a finite planar set,1972.
%   Input: 
    %   lstp: a list of points,  
%   Output: 
    %   y: list of points belonging to the convex hull
function y=graham_scan(lstp) 
    n=size(lstp,1);
    eps=10^(-14);
    if n == 2  
        y=lstp;  
        return;
    end    
    direction=-1;
    stackpointer=1;
    pstack(stackpointer,:)=lstp(1,:); 
    firstp=lstp(2,:);
    secondp=lstp(1,:);
    for i= 3:size(lstp,1) 
        turnp=((lstp(i,1)-secondp(1))*(firstp(2)-secondp(2)))-((firstp(1)-secondp(1))*(lstp(i,2)-secondp(2))); 
        while ((turnp*direction)>0) & (stackpointer>1)  
            pstack(stackpointer,:)=[-1,-1];
            stackpointer=stackpointer-1;
            firstp=secondp;
            secondp=pstack(stackpointer,:);
            turnp=(lstp(i,1)-secondp(1))*(firstp(2)-secondp(2))-(firstp(1)-secondp(1))*(lstp(i,2)-secondp(2));             
        end 
        if ((turnp*direction)>0) & (stackpointer==1)
            firstp=lstp(i,:);  
        else 
            stackpointer=stackpointer+1;           
            pstack(stackpointer,:)=firstp;
            secondp=firstp;
            firstp=lstp(i,:); 
        end
    end
    stackpointer=stackpointer+1;           
    pstack(stackpointer,:)=firstp;
    j=stackpointer;
    while stackpointer~=0 
        y(j,:)=pstack(stackpointer,:); 
        j=j-1;
        pstack(stackpointer,:)=[-1,-1];
        stackpointer=stackpointer-1;           
    end
    pstack=[];
    pstack=y;
    y=[];
    y(1,:)=pstack(1,:); 
    j=2; 
    n=size(pstack,1);
    for i= 2:(n-1)    
        if abs(((pstack(i,2)-pstack((i-1),2))*(pstack((i+1),1)-pstack(i,1)))-((pstack((i+1),2)-pstack(i,2))*(pstack(i,1)-pstack((i-1),1))))>eps  
            y(j,:)=pstack(i,:); 
            j=j+1;
        end,
    end,  
    y(j,:)=pstack(n,:);   
    
end
